@@ -1,21 +1,25 @@
 from __future__ import unicode_literals
 
-from datetime import date, timedelta
-from dateutil.relativedelta import relativedelta
 import math
 import operator
 import re
+from datetime import date
+from logging import getLogger
 
+import six
+from dateutil.relativedelta import relativedelta
 from django.core.exceptions import ValidationError
 from django.db import models
-from django.utils import formats
 from django.utils.dates import MONTHS
-from django.utils.text import capfirst
-import six
 
-from django_easyfilters.queries import date_aggregation, value_counts, numeric_range_counts
-from django_easyfilters.ranges import auto_ranges
-from django_easyfilters.utils import python_2_unicode_compatible, get_model_field
+from .queries import date_aggregation
+from .queries import numeric_range_counts
+from .queries import value_counts
+from .ranges import auto_ranges
+from .utils import get_model_field
+from .utils import python_2_unicode_compatible
+
+logger = getLogger(__name__)
 
 
 if six.PY3:
@@ -50,7 +54,7 @@ class Filter(object):
 
     ### Public interface ###
 
-    def __init__(self, field, model, params, query_param=None, order_by_count=False):
+    def __init__(self, field, model, params, query_param=None, order_by_count=False, sticky=False, show_counts=True):
         self.field = field
         self.model = model
         self.params = params
@@ -65,6 +69,8 @@ class Filter(object):
             self.rel_field = self.field_obj.rel.get_related_field()
         # Make chosen an immutable sequence, to stop accidental mutation.
         self.chosen = tuple(self.choices_from_params())
+        self.sticky = sticky
+        self.show_counts = show_counts
 
     def apply_filter(self, qs):
         """
@@ -74,6 +80,8 @@ class Filter(object):
         chosen = list(self.chosen)
         while len(chosen) > 0:
             lookup = self.lookup_from_choice(chosen.pop())
+            if self.sticky:
+                qs._next_is_sticky()
             qs = qs.filter(**lookup)
         return qs
 
@@ -168,7 +176,7 @@ class Filter(object):
         choices = []
         for choice in chosen:
             choices.append(FilterChoice(self.render_choice_object(choice),
-                                        None, # Don't need count for removing
+                                        None,  # Don't need count for removing
                                         self.build_params(remove=[choice]),
                                         FILTER_REMOVE))
         return choices
@@ -258,7 +266,10 @@ class SimpleQueryMixin(object):
         The order is the underlying order produced by sorting ascending on the
         DB field.
         """
-        return value_counts(qs, self.field)
+        if self.show_counts or self.order_by_count:
+            return value_counts(qs, self.field)
+        else:
+            return dict((val, None) for val, in qs.values_list(self.field).order_by(self.field).distinct())
 
 
 class RangeFilterMixin(ChooseAgainMixin):
@@ -501,6 +512,7 @@ MONTH       = DateRangeType(2, True,  'month', _ym)
 DAYGROUP    = DateRangeType(3, False, 'day',   _ymd)
 DAY         = DateRangeType(3, True,  'day',   _ymd)
 
+
 class NullChoice(object):
     def make_lookup(self, field_name):
         return {field_name+"__isnull": True}
@@ -518,6 +530,24 @@ class NullChoice(object):
     range_type = values = None
 NullChoice = NullChoice()
 
+
+class AnyChoice(object):
+    def make_lookup(self, field_name):
+        return {}
+
+    def display(self):
+        return "(any)"
+    __str__ = __repr__ = display
+
+    def __cmp__(self, other):
+        return 0 if other is AnyChoice else 1
+
+    def __eq__(self, other):
+        return other is AnyChoice
+
+    range_type = values = None
+AnyChoice = AnyChoice()
+
 @python_2_unicode_compatible
 @total_ordering
 class DateChoice(object):
@@ -722,7 +752,7 @@ class DateTimeFilter(RangeFilterMixin, Filter):
         null_count = not chosen and qs.filter(**{self.field + '__isnull': True}).count()
         if null_count:
             choices.append(FilterChoice(self.render_choice_object(NullChoice),
-                                        null_count,
+                                        null_count if self.show_counts else None,
                                         self.build_params(add=NullChoice),
                                         FILTER_ADD))
 
@@ -743,7 +773,7 @@ class DateTimeFilter(RangeFilterMixin, Filter):
                 link_type = FILTER_ADD
 
             choices.append(FilterChoice(self.render_choice_object(date_choice),
-                                        count,
+                                        count if self.show_counts else None,
                                         self.build_params(add=date_choice),
                                         link_type))
         return choices
@@ -839,7 +869,6 @@ class RangeEnd(object):
         self.value, self.inclusive = value, inclusive
 
 
-
 def make_numeric_range_choice(to_python, to_str):
     """
     Returns a Choice class that represents a numeric choice range,
@@ -961,16 +990,15 @@ class NumericRangeFilter(RangeFilterMixin, SingleValueMixin, Filter):
             for v, count in val_counts.items():
                 choice = NullChoice if v is None else self.choice_type([RangeEnd(v, True)])
                 choices.append(FilterChoice(self.render_choice_object(choice),
-                                            count,
+                                            count if self.show_counts else None,
                                             self.build_params(add=choice),
                                             FILTER_ADD))
         else:
             null_count = not chosen and qs.filter(**{self.field + '__isnull': True}).count()
             if null_count:
                 choice = NullChoice
-                #choice = self.choice_type([RangeEnd(None, False)])
                 choices.append(FilterChoice(self.render_choice_object(choice),
-                                            null_count,
+                                            null_count if self.show_counts else None,
                                             self.build_params(add=choice),
                                             FILTER_ADD))
             if self.ranges is None:
@@ -982,7 +1010,10 @@ class NumericRangeFilter(RangeFilterMixin, SingleValueMixin, Filter):
             else:
                 ranges = self.ranges
 
-            val_counts = numeric_range_counts(qs, self.field, ranges)
+            if self.show_counts or self.order_by_count:
+                val_counts = numeric_range_counts(qs, self.field, ranges)
+            else:
+                val_counts = dict((val, None) for val in ranges)
             for i, (vals, count) in enumerate(val_counts.items()):
                 # For the lower bound, we make it inclusive only if it the first
                 # choice. The upper bound is always inclusive. This gives
